classdef layer < handle
    %LAYER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (SetAccess = private)
        id        
        material
    end
    properties (SetAccess = protected)
        type
        wavenumber
        specific_impedance
        transfer_mx
        scattering_mx
        reflection_mx
    end
    
    methods
        function obj = layer( id, material )
            obj.id  = id;
            obj.material = material;
        end
        
    end
    
    methods (Abstract)
        calculate_acoustic_properties()
        calculate_transfer_matrix()
    end
    methods
        function Z = get_impedance(obj)
            Z = obj.specific_impedance;
        end
        function k = get_wavenumber(obj)
            k = obj.wavenumber;
        end
        function T = get_transfer_matrix(obj)
            T = obj.transfer_mx;
        end
        function R = get_reflection_matrix(obj)
            R = obj.reflection_mx;
        end
        
        function calculate_scattering_matrix(obj,fi0)
            obj.calculate_transfer_matrix(fi0);
            Y = reshape(cos(fi0).*obj.specific_impedance,[1,1,length(obj.specific_impedance)]);
            Xp = obj.transfer_mx(1,1,:) + Y.*obj.transfer_mx(1,2,:);
            Xn = obj.transfer_mx(1,1,:) - Y.*obj.transfer_mx(1,2,:);
            Wp = obj.transfer_mx(2,1,:)./Y + obj.transfer_mx(2,2,:);
            Wn = obj.transfer_mx(2,1,:)./Y - obj.transfer_mx(2,2,:);
            obj.scattering_mx = zeros(size(obj.transfer_mx));
            obj.scattering_mx(1,1,:) = (Xp - Wp)./(Xp + Wp);
            obj.scattering_mx(1,2,:) = (Xp.*Wn - Wp.*Xn)./(Xp + Wp);
            obj.scattering_mx(2,1,:) = 2./(Xp + Wp);
            obj.scattering_mx(2,2,:) = -(Xn + Wn)./(Xp + Wp);
        end
        function calculate_reflection_matrix(obj,fi0)
            obj.calculate_transfer_matrix(fi0);
            obj.calculate_scattering_matrix(fi0);
            obj.reflection_mx = zeros(size(obj.transfer_mx));
            
            D11 = obj.scattering_mx(1,1,:);
            D12 = obj.scattering_mx(1,2,:);
            D21 = obj.scattering_mx(2,1,:);
            D22 = obj.scattering_mx(2,2,:);
            obj.reflection_mx(1,1,:) = 1./D21;
            obj.reflection_mx(2,1,:) = -D22;
            obj.reflection_mx(1,2,:) = D11./D21;
            obj.reflection_mx(2,2,:) = D12 - D11.*D22./D21;
         end
        
    end
end

