classdef perforated_panel < layer
    %SLOTTED_PLATE Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (SetAccess = private)
        thickness
        dthickness % thickness_correction
        D    % repeat_distance (spacing)
        r    % hole radius
    end
    
    methods
        function obj = perforated_panel(id, material, thickness, D, r)
            %LIMP_MEMBRANE_LAYER Construct an instance of this class
            %   Detailed explanation goes here
            obj = obj@layer(id, material);
            obj.type = 'perforated_panel';
            obj.thickness = thickness;
            obj.D = D;
            obj.r = r;
        end
        
        function calculate_acoustic_properties(obj,env,f)
            obj.wavenumber = 2*pi*f/env.get_speedofsound;
            eps = pi*obj.r^2 / obj.D^2;
            v = 15e-6;
            delta = 0.85; % See Cox, p243
            m = env.get_density/eps*(obj.thickness + 2*delta*obj.r + ...
                sqrt(8*v./(2*pi*f).*(1 + obj.thickness/(2*obj.r))  ));
            r = env.get_density/eps*sqrt(8*v*2*pi*f)*(1 + obj.thickness/obj.r);
            obj.specific_impedance = r + 1i*2*pi*f.*m;
        end
        
        function obj = calculate_transfer_matrix(obj, fi0)
            Z0 = reshape(obj.specific_impedance,[1,1,length(obj.specific_impedance)]);
            obj.transfer_mx = [ones(1,1,length(Z0)), Z0;
                               zeros(1,1,length(Z0)),ones(1,1,length(Z0))];
        end

    end
end

