classdef porous_layer < layer
    %POROUS_LAYER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (SetAccess = private)
        thickness
        flow_resistivity
        porosity
        model
    end
    
    methods
        function obj = porous_layer(id, material, thickness, flow_resistivity, model)
            obj = obj@layer(id,material);
            obj.thickness = thickness;
            obj.type = 'porous_layer';
            obj.flow_resistivity = flow_resistivity;
            obj.model = model;
            obj.porosity = 0.99;
        end
        
        function calculate_acoustic_properties(obj,env,f)
            switch obj.model
                case 'delany'
                    X = env.get_density*f/obj.flow_resistivity;
                    obj.specific_impedance = env.get_impedance*( 1 + 0.0571*X.^(-0.754) - 1i*0.087*X.^(-0.732));
                    obj.wavenumber = (2*pi*f)./env.get_speedofsound.*( 1 + 0.0978*X.^(-0.7) - 1i*0.189*X.^(-0.595));
            end
        end
        
        function calculate_transfer_matrix(obj,fi0)
            d = obj.thickness;
            Z0 = reshape(obj.specific_impedance,[1,1,length(obj.specific_impedance)]);
            kz = reshape(obj.wavenumber*cos(fi0),[1,1,length(obj.wavenumber)]);
            obj.transfer_mx = [ cos(kz.*d),               1i*Z0.*sin(kz.*d)./cos(fi0);
                1i*sin(kz.*d).*cos(fi0)./Z0,      cos(kz.*d)];
            ap_transition = repmat([1 0;0 obj.porosity],[1,1,size(obj.transfer_mx,3)]);
            pa_transition = repmat([1 0;0 1./obj.porosity],[1,1,size(obj.transfer_mx,3)]);
            obj.transfer_mx = MatrixMult(MatrixMult(ap_transition,obj.transfer_mx),pa_transition);
        end
        
    end
end

